Mat2

Struct Mat2 

pub struct Mat2(/* private fields */);
A 2x2 column major matrix.

SIMD vector types are used for storage on supported platforms.

This type is 16 byte aligned.

Implementations

Source

impl Mat2

pub const ZERO: Mat2

A 2x2 matrix with all elements set to 0.0.

pub const IDENTITY: Mat2

A 2x2 identity matrix, where all diagonal elements are 1, and all off-diagonal elements are 0.

pub const NAN: Mat2

All NAN:s.

pub const fn from_cols(x_axis: Vec2, y_axis: Vec2) -> Mat2

Creates a 2x2 matrix from two column vectors.

pub const fn from_cols_array(m: &[f32; 4]) -> Mat2

Creates a 2x2 matrix from a [f32; 4] array stored in column major order. If your data is stored in row major you will need to transpose the returned matrix.

pub const fn to_cols_array(&self) -> [f32; 4]

Creates a [f32; 4] array storing data in column major order. If you require data in row major order transpose the matrix first.

pub const fn from_cols_array_2d(m: &[[f32; 2]; 2]) -> Mat2

Creates a 2x2 matrix from a [[f32; 2]; 2] 2D array stored in column major order. If your data is in row major order you will need to transpose the returned matrix.

pub const fn to_cols_array_2d(&self) -> [[f32; 2]; 2]

Creates a [[f32; 2]; 2] 2D array storing data in column major order. If you require data in row major order transpose the matrix first.

pub const fn from_diagonal(diagonal: Vec2) -> Mat2

Creates a 2x2 matrix with its diagonal set to diagonal and all other entries set to 0.

pub fn from_scale_angle(scale: Vec2, angle: f32) -> Mat2

Creates a 2x2 matrix containing the combining non-uniform scale and rotation of angle (in radians).

pub fn from_angle(angle: f32) -> Mat2

Creates a 2x2 matrix containing a rotation of angle (in radians).

pub fn from_mat3(m: Mat3) -> Mat2

Creates a 2x2 matrix from a 3x3 matrix, discarding the 2nd row and column.

pub fn from_mat3_minor(m: Mat3, i: usize, j: usize) -> Mat2

Creates a 2x2 matrix from the minor of the given 3x3 matrix, discarding the ith column and jth row.

Panics

Panics if i or j is greater than 2.

pub fn from_mat3a(m: Mat3A) -> Mat2

Creates a 2x2 matrix from a 3x3 matrix, discarding the 2nd row and column.

pub fn from_mat3a_minor(m: Mat3A, i: usize, j: usize) -> Mat2

Creates a 2x2 matrix from the minor of the given 3x3 matrix, discarding the ith column and jth row.

Panics

Panics if i or j is greater than 2.

pub const fn from_cols_slice(slice: &[f32]) -> Mat2

Creates a 2x2 matrix from the first 4 values in slice.

Panics

Panics if slice is less than 4 elements long.

pub fn write_cols_to_slice(&self, slice: &mut [f32])

Writes the columns of self to the first 4 elements in slice.

Panics

Panics if slice is less than 4 elements long.

pub fn col(&self, index: usize) -> Vec2

Returns the matrix column for the given index.

Panics

Panics if index is greater than 1.

pub fn col_mut(&mut self, index: usize) -> &mut Vec2

Returns a mutable reference to the matrix column for the given index.

Panics

Panics if index is greater than 1.

pub fn row(&self, index: usize) -> Vec2

Returns the matrix row for the given index.

Panics

Panics if index is greater than 1.

pub fn is_finite(&self) -> bool

Returns true if, and only if, all elements are finite. If any element is either NaN, positive or negative infinity, this will return false.

pub fn is_nan(&self) -> bool

Returns true if any elements are NaN.

pub fn transpose(&self) -> Mat2

Returns the transpose of self.

pub fn diagonal(&self) -> Vec2

Returns the diagonal of self.

pub fn determinant(&self) -> f32

Returns the determinant of self.

pub fn inverse(&self) -> Mat2

Returns the inverse of self.

If the matrix is not invertible the returned matrix will be invalid.

Panics

Will panic if the determinant of self is zero when glam_assert is enabled.

pub fn try_inverse(&self) -> Option<Mat2>

Returns the inverse of self or None if the matrix is not invertible.

pub fn inverse_or_zero(&self) -> Mat2

Returns the inverse of self or Mat2::ZERO if the matrix is not invertible.

pub fn mul_vec2(&self, rhs: Vec2) -> Vec2

Transforms a 2D vector.

pub fn mul_transpose_vec2(&self, rhs: Vec2) -> Vec2

Transforms a 2D vector by the transpose of self.

pub fn mul_mat2(&self, rhs: &Mat2) -> Mat2

Multiplies two 2x2 matrices.

pub fn add_mat2(&self, rhs: &Mat2) -> Mat2

Adds two 2x2 matrices.

pub fn sub_mat2(&self, rhs: &Mat2) -> Mat2

Subtracts two 2x2 matrices.

pub fn mul_scalar(&self, rhs: f32) -> Mat2

Multiplies a 2x2 matrix by a scalar.

pub fn mul_diagonal_scale(&self, scale: Vec2) -> Mat2

Multiply self by a scaling vector scale. This is faster than creating a whole diagonal scaling matrix and then multiplying that. This operation is commutative.

pub fn div_scalar(&self, rhs: f32) -> Mat2

Divides a 2x2 matrix by a scalar.

pub fn recip(&self) -> Mat2

Returns a matrix containing the reciprocal 1.0/n of each element of self.

pub fn abs_diff_eq(&self, rhs: Mat2, max_abs_diff: f32) -> bool

Returns true if the absolute difference of all elements between self and rhs is less than or equal to max_abs_diff.

This can be used to compare if two matrices contain similar elements. It works best when comparing with a known value. The max_abs_diff that should be used used depends on the values being compared against.

For more see comparing floating point numbers.

pub fn abs(&self) -> Mat2

Takes the absolute value of each element in self

pub fn as_dmat2(&self) -> DMat2