Vec3

Struct Vec3 

#[repr(C)]
pub struct Vec3 { pub x: f32, pub y: f32, pub z: f32, }
A 3-dimensional vector.

Fields

x: f32y: f32z: f32

Implementations

Source

impl Vec3

pub const ZERO: Vec3

All zeroes.

pub const ONE: Vec3

All ones.

pub const NEG_ONE: Vec3

All negative ones.

pub const MIN: Vec3

All f32::MIN.

pub const MAX: Vec3

All f32::MAX.

pub const NAN: Vec3

All f32::NAN.

pub const INFINITY: Vec3

All f32::INFINITY.

pub const NEG_INFINITY: Vec3

All f32::NEG_INFINITY.

pub const X: Vec3

A unit vector pointing along the positive X axis.

pub const Y: Vec3

A unit vector pointing along the positive Y axis.

pub const Z: Vec3

A unit vector pointing along the positive Z axis.

pub const NEG_X: Vec3

A unit vector pointing along the negative X axis.

pub const NEG_Y: Vec3

A unit vector pointing along the negative Y axis.

pub const NEG_Z: Vec3

A unit vector pointing along the negative Z axis.

pub const AXES: [Vec3; 3]

The unit axes.

pub const USES_CORE_SIMD: bool = false

Vec3 uses Rust Portable SIMD

pub const USES_NEON: bool = false

Vec3 uses Arm NEON

pub const USES_SCALAR_MATH: bool = true

Vec3 uses scalar math

pub const USES_SSE2: bool = false

Vec3 uses Intel SSE2

pub const USES_WASM_SIMD: bool = false

Vec3 uses WebAssembly 128-bit SIMD

pub const USES_WASM32_SIMD: bool = false

👎Deprecated since 0.31.0:

Renamed to USES_WASM_SIMD

pub const fn new(x: f32, y: f32, z: f32) -> Vec3

Creates a new vector.

pub const fn splat(v: f32) -> Vec3

Creates a vector with all elements set to v.

pub fn map<F>(self, f: F) -> Vec3
where F: Fn(f32) -> f32,

Returns a vector containing each element of self modified by a mapping function f.

pub fn select(mask: BVec3, if_true: Vec3, if_false: Vec3) -> Vec3

Creates a vector from the elements in if_true and if_false, selecting which to use for each element of self.

A true element in the mask uses the corresponding element from if_true, and false uses the element from if_false.

pub const fn from_array(a: [f32; 3]) -> Vec3

Creates a new vector from an array.

pub const fn to_array(&self) -> [f32; 3]

Converts self to [x, y, z]

pub const fn from_slice(slice: &[f32]) -> Vec3

Creates a vector from the first 3 values in slice.

Panics

Panics if slice is less than 3 elements long.

pub fn write_to_slice(self, slice: &mut [f32])

Writes the elements of self to the first 3 elements in slice.

Panics

Panics if slice is less than 3 elements long.

pub fn extend(self, w: f32) -> Vec4

Creates a 4D vector from self and the given w value.

pub fn truncate(self) -> Vec2

Creates a 2D vector from the x and y elements of self, discarding z.

Truncation may also be performed by using self.xy().

pub fn from_homogeneous(v: Vec4) -> Vec3

Projects a homogeneous coordinate to 3D space by performing perspective divide.

Panics

Will panic if v.w is 0 when glam_assert is enabled.

pub fn to_homogeneous(self) -> Vec4

Creates a homogeneous coordinate from self, equivalent to self.extend(1.0).

pub fn to_vec3a(self) -> Vec3A

pub fn with_x(self, x: f32) -> Vec3

Creates a 3D vector from self with the given value of x.

pub fn with_y(self, y: f32) -> Vec3

Creates a 3D vector from self with the given value of y.

pub fn with_z(self, z: f32) -> Vec3

Creates a 3D vector from self with the given value of z.

pub fn dot(self, rhs: Vec3) -> f32

Computes the dot product of self and rhs.

pub fn dot_into_vec(self, rhs: Vec3) -> Vec3

Returns a vector where every component is the dot product of self and rhs.

pub fn cross(self, rhs: Vec3) -> Vec3

Computes the cross product of self and rhs.

pub fn min(self, rhs: Vec3) -> Vec3

Returns a vector containing the minimum values for each element of self and rhs.

In other words this computes [min(x, rhs.x), min(self.y, rhs.y), ..].

NaN propogation does not follow IEEE 754-2008 semantics for minNum and may differ on different SIMD architectures.

pub fn max(self, rhs: Vec3) -> Vec3

Returns a vector containing the maximum values for each element of self and rhs.

In other words this computes [max(self.x, rhs.x), max(self.y, rhs.y), ..].

NaN propogation does not follow IEEE 754-2008 semantics for maxNum and may differ on different SIMD architectures.

pub fn clamp(self, min: Vec3, max: Vec3) -> Vec3

Component-wise clamping of values, similar to f32::clamp.

Each element in min must be less-or-equal to the corresponding element in max.

NaN propogation does not follow IEEE 754-2008 semantics and may differ on different SIMD architectures.

Panics

Will panic if min is greater than max when glam_assert is enabled.

pub fn min_element(self) -> f32

Returns the horizontal minimum of self.

In other words this computes min(x, y, ..).

NaN propogation does not follow IEEE 754-2008 semantics and may differ on different SIMD architectures.

pub fn max_element(self) -> f32

Returns the horizontal maximum of self.

In other words this computes max(x, y, ..).

NaN propogation does not follow IEEE 754-2008 semantics and may differ on different SIMD architectures.

pub fn min_position(self) -> usize

Returns the index of the first minimum element of self.

pub fn max_position(self) -> usize

Returns the index of the first maximum element of self.

pub fn element_sum(self) -> f32

Returns the sum of all elements of self.

In other words, this computes self.x + self.y + ...

pub fn element_product(self) -> f32

Returns the product of all elements of self.

In other words, this computes self.x * self.y * ...

pub fn cmpeq(self, rhs: Vec3) -> BVec3

Returns a vector mask containing the result of a == comparison for each element of self and rhs.

In other words, this computes [self.x == rhs.x, self.y == rhs.y, ..] for all elements.

pub fn cmpne(self, rhs: Vec3) -> BVec3

Returns a vector mask containing the result of a != comparison for each element of self and rhs.

In other words this computes [self.x != rhs.x, self.y != rhs.y, ..] for all elements.

pub fn cmpge(self, rhs: Vec3) -> BVec3

Returns a vector mask containing the result of a >= comparison for each element of self and rhs.

In other words this computes [self.x >= rhs.x, self.y >= rhs.y, ..] for all elements.

pub fn cmpgt(self, rhs: Vec3) -> BVec3

Returns a vector mask containing the result of a > comparison for each element of self and rhs.

In other words this computes [self.x > rhs.x, self.y > rhs.y, ..] for all elements.

pub fn cmple(self, rhs: Vec3) -> BVec3

Returns a vector mask containing the result of a <= comparison for each element of self and rhs.

In other words this computes [self.x <= rhs.x, self.y <= rhs.y, ..] for all elements.

pub fn cmplt(self, rhs: Vec3) -> BVec3

Returns a vector mask containing the result of a < comparison for each element of self and rhs.

In other words this computes [self.x < rhs.x, self.y < rhs.y, ..] for all elements.

pub fn abs(self) -> Vec3

Returns a vector containing the absolute value of each element of self.

pub fn signum(self) -> Vec3

Returns a vector with elements representing the sign of self.

  • 1.0 if the number is positive, +0.0 or INFINITY
  • -1.0 if the number is negative, -0.0 or NEG_INFINITY
  • NAN if the number is NAN
pub fn copysign(self, rhs: Vec3) -> Vec3

Returns a vector with signs of rhs and the magnitudes of self.

pub fn is_negative_bitmask(self) -> u32

Returns a bitmask with the lowest 3 bits set to the sign bits from the elements of self.

A negative element results in a 1 bit and a positive element in a 0 bit. Element x goes into the first lowest bit, element y into the second, etc.

An element is negative if it has a negative sign, including -0.0, NaNs with negative sign bit and negative infinity.

pub fn is_finite(self) -> bool

Returns true if, and only if, all elements are finite. If any element is either NaN, positive or negative infinity, this will return false.

pub fn is_finite_mask(self) -> BVec3

Performs is_finite on each element of self, returning a vector mask of the results.

In other words, this computes [x.is_finite(), y.is_finite(), ...].

pub fn is_nan(self) -> bool

Returns true if any elements are NaN.

pub fn is_nan_mask(self) -> BVec3

Performs is_nan on each element of self, returning a vector mask of the results.

In other words, this computes [x.is_nan(), y.is_nan(), ...].

pub fn length(self) -> f32

Computes the length of self.

pub fn length_squared(self) -> f32

Computes the squared length of self.

This is faster than length() as it avoids a square root operation.

pub fn length_recip(self) -> f32

Computes 1.0 / length().

For valid results, self must not be of length zero.

pub fn distance(self, rhs: Vec3) -> f32

Computes the Euclidean distance between two points in space.

pub fn distance_squared(self, rhs: Vec3) -> f32

Compute the squared euclidean distance between two points in space.

pub fn div_euclid(self, rhs: Vec3) -> Vec3

Returns the element-wise quotient of [Euclidean division] of self by rhs.

pub fn rem_euclid(self, rhs: Vec3) -> Vec3

Returns the element-wise remainder of Euclidean division of self by rhs.

pub fn normalize(self) -> Vec3

Returns self normalized to length 1.0.

For valid results, self must be finite and not of length zero, nor very close to zero.

See also Self::try_normalize() and Self::normalize_or_zero().

Panics

Will panic if the resulting normalized vector is not finite when glam_assert is enabled.

pub fn try_normalize(self) -> Option<Vec3>

Returns self normalized to length 1.0 if possible, else returns None.

In particular, if the input is zero (or very close to zero), or non-finite, the result of this operation will be None.

See also Self::normalize_or_zero().

pub fn normalize_or(self, fallback: Vec3) -> Vec3

Returns self normalized to length 1.0 if possible, else returns a fallback value.

In particular, if the input is zero (or very close to zero), or non-finite, the result of this operation will be the fallback value.

See also Self::try_normalize().

pub fn normalize_or_zero(self) -> Vec3

Returns self normalized to length 1.0 if possible, else returns zero.

In particular, if the input is zero (or very close to zero), or non-finite, the result of this operation will be zero.

See also Self::try_normalize().

pub fn normalize_and_length(self) -> (Vec3, f32)

Returns self normalized to length 1.0 and the length of self.

If self is zero length then (Self::X, 0.0) is returned.

pub fn is_normalized(self) -> bool

Returns whether self is length 1.0 or not.

Uses a precision threshold of approximately 1e-4.

pub fn project_onto(self, rhs: Vec3) -> Vec3

Returns the vector projection of self onto rhs.

rhs must be of non-zero length.

Panics

Will panic if rhs is zero length when glam_assert is enabled.

pub fn reject_from(self, rhs: Vec3) -> Vec3

Returns the vector rejection of self from rhs.

The vector rejection is the vector perpendicular to the projection of self onto rhs, in rhs words the result of self - self.project_onto(rhs).

rhs must be of non-zero length.

Panics

Will panic if rhs has a length of zero when glam_assert is enabled.

pub fn project_onto_normalized(self, rhs: Vec3) -> Vec3

Returns the vector projection of self onto rhs.

rhs must be normalized.

Panics

Will panic if rhs is not normalized when glam_assert is enabled.

pub fn reject_from_normalized(self, rhs: Vec3) -> Vec3

Returns the vector rejection of self from rhs.

The vector rejection is the vector perpendicular to the projection of self onto rhs, in rhs words the result of self - self.project_onto(rhs).

rhs must be normalized.

Panics

Will panic if rhs is not normalized when glam_assert is enabled.

pub fn round(self) -> Vec3

Returns a vector containing the nearest integer to a number for each element of self. Round half-way cases away from 0.0.

pub fn floor(self) -> Vec3

Returns a vector containing the largest integer less than or equal to a number for each element of self.

pub fn ceil(self) -> Vec3

Returns a vector containing the smallest integer greater than or equal to a number for each element of self.

pub fn trunc(self) -> Vec3

Returns a vector containing the integer part each element of self. This means numbers are always truncated towards zero.

pub fn step(self, rhs: Vec3) -> Vec3

Returns a vector containing 0.0 if rhs < self and 1.0 otherwise.

Similar to glsl’s step(edge, x), which translates into edge.step(x)

pub fn saturate(self) -> Vec3

Returns a vector containing all elements of self clamped to the range of [0, 1].

pub fn fract(self) -> Vec3

Returns a vector containing the fractional part of the vector as self - self.trunc().

Note that this differs from the GLSL implementation of fract which returns self - self.floor().

Note that this is fast but not precise for large numbers.

pub fn fract_gl(self) -> Vec3

Returns a vector containing the fractional part of the vector as self - self.floor().

Note that this differs from the Rust implementation of fract which returns self - self.trunc().

Note that this is fast but not precise for large numbers.

pub fn exp(self) -> Vec3

Returns a vector containing e^self (the exponential function) for each element of self.

pub fn exp2(self) -> Vec3

Returns a vector containing 2^self for each element of self.

pub fn ln(self) -> Vec3

Returns a vector containing the natural logarithm for each element of self. This returns NaN when the element is negative and negative infinity when the element is zero.

pub fn log2(self) -> Vec3

Returns a vector containing the base 2 logarithm for each element of self. This returns NaN when the element is negative and negative infinity when the element is zero.

pub fn powf(self, n: f32) -> Vec3

Returns a vector containing each element of self raised to the power of n.

pub fn sqrt(self) -> Vec3

Returns a vector containing the square root for each element of self. This returns NaN when the element is negative.

pub fn cos(self) -> Vec3

Returns a vector containing the cosine for each element of self.

pub fn sin(self) -> Vec3

Returns a vector containing the sine for each element of self.

pub fn sin_cos(self) -> (Vec3, Vec3)

Returns a tuple of two vectors containing the sine and cosine for each element of self.

pub fn recip(self) -> Vec3

Returns a vector containing the reciprocal 1.0/n of each element of self.

pub fn lerp(self, rhs: Vec3, s: f32) -> Vec3

Performs a linear interpolation between self and rhs based on the value s.

When s is 0.0, the result will be equal to self. When s is 1.0, the result will be equal to rhs. When s is outside of range [0, 1], the result is linearly extrapolated.

pub fn move_towards(self, rhs: Vec3, d: f32) -> Vec3

Moves towards rhs based on the value d.

When d is 0.0, the result will be equal to self. When d is equal to self.distance(rhs), the result will be equal to rhs. Will not go past rhs.

pub fn midpoint(self, rhs: Vec3) -> Vec3

Calculates the midpoint between self and rhs.

The midpoint is the average of, or halfway point between, two vectors. a.midpoint(b) should yield the same result as a.lerp(b, 0.5) while being slightly cheaper to compute.

pub fn abs_diff_eq(self, rhs: Vec3, max_abs_diff: f32) -> bool

Returns true if the absolute difference of all elements between self and rhs is less than or equal to max_abs_diff.

This can be used to compare if two vectors contain similar elements. It works best when comparing with a known value. The max_abs_diff that should be used used depends on the values being compared against.

For more see comparing floating point numbers.

pub fn clamp_length(self, min: f32, max: f32) -> Vec3

Returns a vector with a length no less than min and no more than max.

Panics

Will panic if min is greater than max, or if either min or max is negative, when glam_assert is enabled.

pub fn clamp_length_max(self, max: f32) -> Vec3

Returns a vector with a length no more than max.

Panics

Will panic if max is negative when glam_assert is enabled.

pub fn clamp_length_min(self, min: f32) -> Vec3

Returns a vector with a length no less than min.

Panics

Will panic if min is negative when glam_assert is enabled.

pub fn mul_add(self, a: Vec3, b: Vec3) -> Vec3

Fused multiply-add. Computes (self * a) + b element-wise with only one rounding error, yielding a more accurate result than an unfused multiply-add.

Using mul_add may be more performant than an unfused multiply-add if the target architecture has a dedicated fma CPU instruction. However, this is not always true, and will be heavily dependant on designing algorithms with specific target hardware in mind.

pub fn reflect(self, normal: Vec3) -> Vec3

Returns the reflection vector for a given incident vector self and surface normal normal.

normal must be normalized.

Panics

Will panic if normal is not normalized when glam_assert is enabled.

pub fn refract(self, normal: Vec3, eta: f32) -> Vec3

Returns the refraction direction for a given incident vector self, surface normal normal and ratio of indices of refraction, eta. When total internal reflection occurs, a zero vector will be returned.

self and normal must be normalized.

Panics

Will panic if self or normal is not normalized when glam_assert is enabled.

pub fn angle_between(self, rhs: Vec3) -> f32

Returns the angle (in radians) between two vectors in the range [0, +π].

The inputs do not need to be unit vectors however they must be non-zero.

pub fn rotate_x(self, angle: f32) -> Vec3

Rotates around the x axis by angle (in radians).

pub fn rotate_y(self, angle: f32) -> Vec3

Rotates around the y axis by angle (in radians).

pub fn rotate_z(self, angle: f32) -> Vec3

Rotates around the z axis by angle (in radians).

pub fn rotate_axis(self, axis: Vec3, angle: f32) -> Vec3

Rotates around axis by angle (in radians).

The axis must be a unit vector.

Panics

Will panic if axis is not normalized when glam_assert is enabled.

pub fn rotate_towards(self, rhs: Vec3, max_angle: f32) -> Vec3

Rotates towards rhs up to max_angle (in radians).

When max_angle is 0.0, the result will be equal to self. When max_angle is equal to self.angle_between(rhs), the result will be parallel to rhs. If max_angle is negative, rotates towards the exact opposite of rhs. Will not go past the target.

pub fn any_orthogonal_vector(self) -> Vec3

Returns some vector that is orthogonal to the given one.

The input vector must be finite and non-zero.

The output vector is not necessarily unit length. For that use Self::any_orthonormal_vector() instead.

pub fn any_orthonormal_vector(self) -> Vec3

Returns any unit vector that is orthogonal to the given one.

The input vector must be unit length.

Panics

Will panic if self is not normalized when glam_assert is enabled.

pub fn any_orthonormal_pair(self) -> (Vec3, Vec3)

Given a unit vector return two other vectors that together form an orthonormal basis. That is, all three vectors are orthogonal to each other and are normalized.

Panics

Will panic if self is not normalized when glam_assert is enabled.

pub fn slerp(self, rhs: Vec3, s: f32) -> Vec3

Performs a spherical linear interpolation between self and rhs based on the value s.

When s is 0.0, the result will be equal to self. When s is 1.0, the result will be equal to rhs. When s is outside of range [0, 1], the result is linearly extrapolated.

pub fn as_dvec3(self) -> DVec3

Casts all elements of self to f64.

pub fn as_i8vec3(self) -> I8Vec3

Casts all elements of self to i8.

pub fn as_u8vec3(self) -> U8Vec3

Casts all elements of self to u8.

pub fn as_i16vec3(self) -> I16Vec3

Casts all elements of self to i16.

pub fn as_u16vec3(self) -> U16Vec3

Casts all elements of self to u16.

pub fn as_ivec3(self) -> IVec3

Casts all elements of self to i32.

pub fn as_uvec3(self) -> UVec3

Casts all elements of self to u32.

pub fn as_i64vec3(self) -> I64Vec3

Casts all elements of self to i64.

pub fn as_u64vec3(self) -> U64Vec3

Casts all elements of self to u64.

pub fn as_isizevec3(self) -> ISizeVec3

Casts all elements of self to isize.

pub fn as_usizevec3(self) -> USizeVec3

Casts all elements of self to usize.