pub enum MappingSource {
SdlMappings,
Driver,
None,
}Expand description
Source of gamepad mappings.
Variants
SdlMappings
Gamepad uses SDL mappings.
Driver
Gamepad does not use any mappings but driver should provide unified controller layout.
None
Gamepad does not use any mappings and most gamepad events will probably be Button::Unknown
or Axis::Unknown