LevelFilter

Enum LevelFilter 

#[repr(usize)]
pub enum LevelFilter { Off = 0, Error = 1, Warn = 2, Info = 3, Debug = 4, Trace = 5, }
An enum representing the available verbosity level filters of the logger.

A LevelFilter may be compared directly to a Level. Use this type to get and set the maximum log level with max_level() and set_max_level.

Variants



Off = 0

A level lower than all log levels.



Error = 1

Corresponds to the Error log level.



Warn = 2

Corresponds to the Warn log level.



Info = 3

Corresponds to the Info log level.



Debug = 4

Corresponds to the Debug log level.



Trace = 5

Corresponds to the Trace log level.

Implementations

Source

impl LevelFilter

pub fn max() -> LevelFilter

Returns the most verbose logging level filter.

Examples found in repository?
examples/logging.rs (line 6)
3fn main() -> anyhow::Result<()> {
4    let opts = EngineCreationOptions::builder()
5        .title("Logging".to_string())
6        .min_log_level(LevelFilter::max())
7        .log_verbosity(Verbosity::Medium)
8        .build();
9
10    init_custom(opts)?;
11    let mut controller = PanningCameraController::new();
12    set_max_drawn_log_lines(50);
13
14    trace!("Hello");
15    debug!("Hello!");
16    info!("Hello!!");
17    warn!("Hello!!!");
18    error!("Hello!!!!");
19    trace!("Hello");
20    debug!("Hello!");
21    info!("Hello!!");
22    warn!("Hello!!!");
23    error!("Hello!!!!");
24    trace!("Hello");
25    debug!("Hello!");
26    info!("Hello!!");
27    warn!("Hello!!!");
28    error!("Hello!!!!");
29    trace!("Hello");
30    debug!("Hello!");
31    info!("Hello!!");
32    warn!("Hello!!!");
33    error!("Hello!!!!");
34
35    run_async(async move {
36        loop {
37            controller.update();
38            draw_logs();
39
40            if once_per_n_seconds(4.0) {
41                info!("im so alone");
42            } else if once_per_n_seconds(2.0) {
43                warn!("haha");
44            } else if once_per_second() {
45                error!("BAHAHAHA");
46            } else if once_per_n_seconds(0.5) {
47                debug!("BAHAHAHAHAHA");
48            }
49
50            let mut font = default_font();
51            let texture = font.texture();
52            draw_texture_world(texture, Vec2::ZERO, 1000.0);
53            draw_square_outline_world(Vec2::ZERO, 1000.0, 2.0, Color::NEUTRAL_500);
54
55            if should_quit() {
56                break;
57            }
58
59            next_frame().await;
60        }
61    });
62
63    Ok(())
64}
pub fn to_level(&self) -> Option<Level>

Converts self to the equivalent Level.

Returns None if self is LevelFilter::Off.

pub fn as_str(&self) -> &'static str

Returns the string representation of the LevelFilter.

This returns the same string as the fmt::Display implementation.

pub fn iter() -> impl Iterator<Item = LevelFilter>

Iterate through all supported filtering levels.

The order of iteration is from less to more verbose filtering.

Examples
use log::LevelFilter;

let mut levels = LevelFilter::iter();

assert_eq!(Some(LevelFilter::Off), levels.next());
assert_eq!(Some(LevelFilter::Trace), levels.last());
pub fn increment_severity(&self) -> LevelFilter

Get the next-highest LevelFilter from this one.

If the current LevelFilter is at the highest level, the returned LevelFilter will be the same as the current one.

Examples
use log::LevelFilter;

let level_filter = LevelFilter::Info;

assert_eq!(LevelFilter::Debug, level_filter.increment_severity());
assert_eq!(LevelFilter::Trace, level_filter.increment_severity().increment_severity());
assert_eq!(LevelFilter::Trace, level_filter.increment_severity().increment_severity().increment_severity()); // max level
pub fn decrement_severity(&self) -> LevelFilter

Get the next-lowest LevelFilter from this one.

If the current LevelFilter is at the lowest level, the returned LevelFilter will be the same as the current one.

Examples
use log::LevelFilter;

let level_filter = LevelFilter::Info;

assert_eq!(LevelFilter::Warn, level_filter.decrement_severity());
assert_eq!(LevelFilter::Error, level_filter.decrement_severity().decrement_severity());
assert_eq!(LevelFilter::Off, level_filter.decrement_severity().decrement_severity().decrement_severity());
assert_eq!(LevelFilter::Off, level_filter.decrement_severity().decrement_severity().decrement_severity().decrement_severity()); // min level