pub struct Gamepad<'a> { /* private fields */ }Expand description
Represents handle to game controller.
Using this struct you can access cached gamepad state, information about gamepad such as name or UUID and manage force feedback effects.
Implementations
Sourceimpl Gamepad<'_>
impl Gamepad<'_>
Sourcepub fn name(&self) -> &str
pub fn name(&self) -> &str
Returns the mapping name if it exists otherwise returns the os provided name.
Examples found in repository?
28fn choose_gamepad() -> Option<GamepadId> {
29 let input = gamepad::input();
30 let gamepads: Vec<_> = input.gamepads().collect();
31
32 use ui::prelude::*;
33 let ui = Col::with_gap(
34 10.0,
35 gamepads
36 .iter()
37 .enumerate()
38 .map(|(i, (_, pad))| flat::Button::primary_text(i, pad.name()))
39 .collect::<Vec<_>>(),
40 );
41
42 draw_ui(ui, Vec2::splat(10.0));
43
44 Some(
45 gamepads
46 .get(*all_elements_interacted_this_frame().first()?)?
47 .0,
48 )
49}Sourcepub fn map_name(&self) -> Option<&str>
pub fn map_name(&self) -> Option<&str>
if mapping_source() is SdlMappings returns the name of the mapping used by the gamepad.
Otherwise returns None.
Sourcepub fn uuid(&self) -> [u8; 16]
pub fn uuid(&self) -> [u8; 16]
Returns gamepad’s UUID.
It is recommended to process with the UUID crate.
Use Uuid::from_bytes method to create a Uuid from the returned bytes.
Sourcepub fn vendor_id(&self) -> Option<u16>
pub fn vendor_id(&self) -> Option<u16>
Returns the vendor ID, as assigned by the USB-IF, when available.
Sourcepub fn product_id(&self) -> Option<u16>
pub fn product_id(&self) -> Option<u16>
Returns the product ID, as assigned by the vendor, when available.
Sourcepub fn state(&self) -> &GamepadState
pub fn state(&self) -> &GamepadState
Returns cached gamepad state.
Sourcepub fn is_connected(&self) -> bool
pub fn is_connected(&self) -> bool
Returns true if gamepad is connected.
Sourcepub fn is_pressed(&self, btn: Button) -> bool
pub fn is_pressed(&self, btn: Button) -> bool
Examines cached gamepad state to check if given button is pressed. Panics if btn is
Unknown.
If you know Code of the element that you want to examine, it’s recommended to use methods
directly on State, because this version have to check which Code is mapped to element of
gamepad.
Examples found in repository?
78fn draw_dpad(pad: Gamepad, center: Vec2) {
79 const SIZE: f32 = 100.0;
80 const THICKNESS: f32 = 65.0;
81 const OUTLINE: f32 = 5.0;
82
83 let half = THICKNESS / 2.0;
84 let half_outline = OUTLINE / 2.0;
85 let arm = SIZE - half;
86
87 let buttons = [
88 (
89 gamepad::Button::DPadUp,
90 vec2(center.x - half, center.y - SIZE),
91 vec2(THICKNESS, arm + half_outline),
92 ),
93 (
94 gamepad::Button::DPadDown,
95 vec2(center.x - half, center.y + half - half_outline),
96 vec2(THICKNESS, arm),
97 ),
98 (
99 gamepad::Button::DPadLeft,
100 vec2(center.x - SIZE, center.y - half),
101 vec2(arm + half_outline, THICKNESS),
102 ),
103 (
104 gamepad::Button::DPadRight,
105 vec2(center.x + half - half_outline, center.y - half),
106 vec2(arm, THICKNESS),
107 ),
108 ];
109
110 let points = [
111 vec2(center.x - half, center.y - SIZE),
112 vec2(center.x + half, center.y - SIZE),
113 vec2(center.x + half, center.y - half),
114 vec2(center.x + SIZE, center.y - half),
115 vec2(center.x + SIZE, center.y + half),
116 vec2(center.x + half, center.y + half),
117 vec2(center.x + half, center.y + SIZE),
118 vec2(center.x - half, center.y + SIZE),
119 vec2(center.x - half, center.y + half),
120 vec2(center.x - SIZE, center.y + half),
121 vec2(center.x - SIZE, center.y - half),
122 vec2(center.x - half, center.y - half),
123 vec2(center.x - half, center.y - SIZE),
124 ];
125
126 draw_connected_path(&points, OUTLINE, SECONDARY);
127
128 for (button, pos, size) in buttons {
129 if pad.is_pressed(button) {
130 draw_rect(pos, size, PRIMARY);
131 }
132 }
133}
134
135fn draw_buttons(pad: Gamepad, center: Vec2) {
136 const RADIUS: f32 = 30.0;
137 const DIST: f32 = 70.0;
138 const OUTLINE: f32 = 5.0;
139
140 let buttons = [
141 (gamepad::Button::North, vec2(center.x, center.y - DIST)),
142 (gamepad::Button::South, vec2(center.x, center.y + DIST)),
143 (gamepad::Button::West, vec2(center.x - DIST, center.y)),
144 (gamepad::Button::East, vec2(center.x + DIST, center.y)),
145 ];
146
147 for (button, pos) in buttons {
148 if pad.is_pressed(button) {
149 draw_circle(pos, RADIUS + OUTLINE, PRIMARY);
150 } else {
151 draw_circle_outline(pos, RADIUS, SECONDARY, OUTLINE);
152 }
153 }
154}Sourcepub fn value(&self, axis: Axis) -> f32
pub fn value(&self, axis: Axis) -> f32
Examines cached gamepad state to check axis’s value. Panics if axis is Unknown.
If you know Code of the element that you want to examine, it’s recommended to use methods
directly on State, because this version have to check which Code is mapped to element of
gamepad.
Returns button state and when it changed.
If you know Code of the element that you want to examine, it’s recommended to use methods
directly on State, because this version have to check which Code is mapped to element of
gamepad.
Sourcepub fn axis_data(&self, axis: Axis) -> Option<&AxisData>
pub fn axis_data(&self, axis: Axis) -> Option<&AxisData>
Returns axis state and when it changed.
If you know Code of the element that you want to examine, it’s recommended to use methods
directly on State, because this version have to check which Code is mapped to element of
gamepad.
Sourcepub fn power_info(&self) -> PowerInfo
pub fn power_info(&self) -> PowerInfo
Returns device’s power supply state. See PowerInfo for details.
Sourcepub fn mapping_source(&self) -> MappingSource
pub fn mapping_source(&self) -> MappingSource
Returns source of gamepad mapping. Can be used to filter gamepads which do not provide unified controller layout.
use gilrs::MappingSource;
for (_, gamepad) in gilrs.gamepads().filter(
|gp| gp.1.mapping_source() != MappingSource::None)
{
println!("{} is ready to use!", gamepad.name());
}Sourcepub fn is_ff_supported(&self) -> bool
pub fn is_ff_supported(&self) -> bool
Returns true if force feedback is supported by device.
Sourcepub fn set_listener_position<Vec3>(&self, position: Vec3) -> Result<(), Error>
pub fn set_listener_position<Vec3>(&self, position: Vec3) -> Result<(), Error>
Change gamepad position used by force feedback effects.
Sourcepub fn axis_or_btn_name(&self, ec: Code) -> Option<AxisOrBtn>
pub fn axis_or_btn_name(&self, ec: Code) -> Option<AxisOrBtn>
Returns AxisOrBtn mapped to Code.
Returns Code associated with btn.