pub fn cursor_diff() -> Vec2Expand description
Returns the change in cursor coordinates that occurred during the last step, when window is focused AND (cursor is on window OR any mouse button remains held while cursor moved off window). Otherwise returns (0.0, 0.0).