pub struct Col {
pub gap: f32,
pub children: Vec<UiRef>,
}Fields
gap: f32children: Vec<UiRef>Implementations
Sourceimpl Col
impl Col
Sourcepub fn new(children: impl Into<Vec<UiRef>>) -> UiRef
pub fn new(children: impl Into<Vec<UiRef>>) -> UiRef
Examples found in repository?
examples/ui.rs (lines 140-166)
133 fn text_section(&self) -> UiRef {
134 BoxFill::new(
135 Color::NEUTRAL_950,
136 Padding::all(
137 20.0,
138 Scroll::new(
139 id!(),
140 Col::new([
141 Text::title("Title"),
142 Text::body(
143 "Lorem ipsum dolor sit amet, consectetur\n\n adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.",
144 ),
145 Text::h1("Heading 1"),
146 Text::body("Lorem ipsum dolor sit amet."),
147 Text::h2("Heading 1"),
148 Text::body("Lorem ipsum dolor sit amet."),
149 Text::mono_sized(
150 format!(
151 "{} {} {} {}",
152 ICON_PLUS_CIRCLE, ICON_BOMB, ICON_CARET_DOWN, ICON_CIRCLE_NOTCH
153 ),
154 40,
155 ),
156 Text::h3("Heading 1"),
157 Text::body("Lorem ipsum dolor sit amet."),
158 Text::italic("Lorem ipsum dolor sit amet."),
159 Text::bold("Lorem ipsum dolor sit amet."),
160 Text::bold_italic("Lorem ipsum dolor sit amet."),
161 Col::new(
162 Palette::NEUTRAL
163 .shades()
164 .map(|c| bold_italic_colored("Lorem ipsum dolor sit amet.", c)),
165 ),
166 ]),
167 ),
168 ),
169 )
170 }More examples
examples/multiplayer.rs (lines 102-126)
69fn chat(state: &mut MultiplayerState<State>) {
70 if !storage_exists::<ChatState>() {
71 storage_store_state(ChatState { messages: vec![] });
72 }
73
74 let messages = &mut storage_get_state_mut::<ChatState>().messages;
75
76 let other_state = unsafe { &*{ state as *const MultiplayerState<State> } };
77 for notification in state.drain_notifications() {
78 let Some(user) = other_state.get_user(notification.user_id) else {
79 break;
80 };
81 let username = &user.username;
82 let text = String::from_utf8(notification.data).unwrap();
83
84 messages.push(rich_text(format!("<b>{username}</b>: {text}")).unwrap());
85 }
86
87 let input_id = id!();
88 let button_id = id!();
89
90 if button_clicked_last_frame(button_id) {
91 let input_state = text_input_state(input_id);
92 messages.push(rich_text(format!("<b>You</b>: {}", input_state.value)).unwrap());
93 state.send_notification(input_state.value.as_bytes().to_vec());
94 input_state.value = "".to_string();
95 }
96
97 let ui = Align::bottom_left(
98 BoxFill::new(
99 flat::BG0,
100 Padding::all(
101 20.0,
102 Col::new([
103 FlexRow::with_gap(
104 10.0,
105 [
106 FlexBox::Flex(flat::TextInput::new(flat::BG1, input_id)),
107 FlexBox::Fixed(flat::Button::text(
108 flat::BG1,
109 flat::BG2,
110 button_id,
111 "Send",
112 )),
113 ],
114 ),
115 Scroll::new(
116 id!(),
117 Col::with_gap(
118 10.0,
119 messages
120 .iter()
121 .map(|r| RichTextNode::new(r.clone()))
122 .collect::<Vec<_>>(),
123 ),
124 )
125 .padding_vertical(20.0),
126 ]),
127 ),
128 )
129 .sized_wh(600.0, 400.0),
130 )
131 .sized(window_size());
132
133 draw_ui(ui, Vec2::ZERO);
134}Sourcepub fn with_gap(gap: f32, children: impl Into<Vec<UiRef>>) -> UiRef
pub fn with_gap(gap: f32, children: impl Into<Vec<UiRef>>) -> UiRef
Examples found in repository?
examples/gamepad.rs (lines 33-40)
28fn choose_gamepad() -> Option<GamepadId> {
29 let input = gamepad::input();
30 let gamepads: Vec<_> = input.gamepads().collect();
31
32 use ui::prelude::*;
33 let ui = Col::with_gap(
34 10.0,
35 gamepads
36 .iter()
37 .enumerate()
38 .map(|(i, (_, pad))| flat::Button::primary_text(i, pad.name()))
39 .collect::<Vec<_>>(),
40 );
41
42 draw_ui(ui, Vec2::splat(10.0));
43
44 Some(
45 gamepads
46 .get(*all_elements_interacted_this_frame().first()?)?
47 .0,
48 )
49}More examples
examples/ui.rs (lines 60-75)
53 fn scroll_section(&self) -> UiRef {
54 BoxFill::new(
55 Color::GRAY_900,
56 Scroll::new(
57 id!(),
58 Padding::all(
59 10.0,
60 Col::with_gap(
61 10.0,
62 (0..100)
63 .map(|n| {
64 SizedBox::height(
65 40.0,
66 Fill::rounded_hover(
67 Color::GRAY_800,
68 Color::GRAY_700,
69 7.0,
70 Center::new(Text::new(n)),
71 ),
72 )
73 })
74 .collect::<Vec<_>>(),
75 ),
76 ),
77 ),
78 )
79 }
80
81 fn w95_section(&mut self) -> UiRef {
82 let color_button = id!();
83 let wait_button = id!();
84
85 if button_clicked_last_frame(color_button) {
86 self.clear_color = rand_color();
87 }
88
89 if button_clicked_last_frame(wait_button) {
90 toggle_wait_for_events();
91 }
92
93 w95::Card::thick(
94 20.0,
95 Scroll::new(
96 id!(),
97 Col::with_gap(
98 10.0,
99 [
100 black_text(format!("{:.0}", avg_fps())),
101 black_text("Hello world!"),
102 black_text("This is UI."),
103 w95::Button::text(color_button, "Background"),
104 w95::Button::text(
105 wait_button,
106 format!("Toggle wait for events: {}", get_wait_for_events()),
107 ),
108 Text::new_with_size_color("Styled text", 30, Color::RED_600),
109 ConstrainedBox::max_size(
110 vec2(300.0, 200.0),
111 w95::Card::new(0.0, ImageNode::from_texture(self.guy)),
112 ),
113 w95::ProgressBar::new(vec2(200.0, 20.0), self.progress, 1.0, id!()),
114 Fill::gradient(
115 Color::NEUTRAL_100,
116 w95::PRIMARY,
117 FRAC_PI_2,
118 Center::new(AspectRatio::new(
119 4.0,
120 BoxFill::new(
121 Color::NEUTRAL_200,
122 CircleFill::new(Color::NEUTRAL_300).sized_wh(150.0, 150.0),
123 ),
124 )),
125 )
126 .sized_wh(200.0, 200.0),
127 ],
128 ),
129 ),
130 )
131 }
132
133 fn text_section(&self) -> UiRef {
134 BoxFill::new(
135 Color::NEUTRAL_950,
136 Padding::all(
137 20.0,
138 Scroll::new(
139 id!(),
140 Col::new([
141 Text::title("Title"),
142 Text::body(
143 "Lorem ipsum dolor sit amet, consectetur\n\n adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.",
144 ),
145 Text::h1("Heading 1"),
146 Text::body("Lorem ipsum dolor sit amet."),
147 Text::h2("Heading 1"),
148 Text::body("Lorem ipsum dolor sit amet."),
149 Text::mono_sized(
150 format!(
151 "{} {} {} {}",
152 ICON_PLUS_CIRCLE, ICON_BOMB, ICON_CARET_DOWN, ICON_CIRCLE_NOTCH
153 ),
154 40,
155 ),
156 Text::h3("Heading 1"),
157 Text::body("Lorem ipsum dolor sit amet."),
158 Text::italic("Lorem ipsum dolor sit amet."),
159 Text::bold("Lorem ipsum dolor sit amet."),
160 Text::bold_italic("Lorem ipsum dolor sit amet."),
161 Col::new(
162 Palette::NEUTRAL
163 .shades()
164 .map(|c| bold_italic_colored("Lorem ipsum dolor sit amet.", c)),
165 ),
166 ]),
167 ),
168 ),
169 )
170 }
171
172 fn button_section(&mut self) -> UiRef {
173 let button = id!();
174
175 if button_clicked_last_frame(button) {
176 self.show_message = !self.show_message;
177 }
178
179 library::flat::Card::bg0_expand(Col::with_gap(
180 30.0,
181 [
182 Center::new(flat::Button::primary_text(button, "Click me!")),
183 if self.show_message {
184 Center::new(Text::body("Thanks for clicking."))
185 } else {
186 EMPTY
187 },
188 ],
189 ))
190 .scissored()
191 }
192
193 fn align_section(&self) -> UiRef {
194 BoxFill::new(
195 Color::BLACK,
196 Stack::new([
197 Align::top_left(square(Color::RED_500)),
198 Align::top_center(square(Color::ORANGE_500)),
199 Align::top_right(square(Color::YELLOW_500)),
200 Align::center_right(square(Color::GREEN_500)),
201 Align::bottom_right(square(Color::SKY_500)),
202 Align::bottom_center(square(Color::BLUE_500)),
203 Align::bottom_left(square(Color::PURPLE_500)),
204 Align::center_left(square(Color::PINK_500)),
205 Align::center(square(Color::WHITE)),
206 ]),
207 )
208 .scissored()
209 }
210
211 fn flat_section(&mut self) -> UiRef {
212 use flat::*;
213
214 let bars: Vec<_> = Palette::PALETTES
215 .iter()
216 .enumerate()
217 .skip(5)
218 .map(|(i, p)| {
219 flat::LoadingBar::new_with_speed(p.v400, i as f32 * 10.0 + 10.0)
220 .height_infinite_width(30.0)
221 })
222 .collect();
223
224 let input_id = id!();
225
226 if text_input_changed(input_id) {
227 println!("{}", text_input_value(input_id));
228 }
229
230 Card::bg0_expand(FlexCol::with_gap(
231 10.0,
232 [
233 FlexBox::Flex(Col::with_gap(
234 20.0,
235 [
236 Drawer::new(
237 "Click me!",
238 BG1,
239 id!(),
240 Col::with_gap(10.0, bars).scroll(id!()),
241 )
242 .max_height(400.0),
243 Text::new(self.slider_a_value),
244 Slider::new(&mut self.slider_a_value, 0, 30, id!()),
245 Slider::alternate(&mut self.slider_b_value, 0.0, 30.0, id!()),
246 Slider::rounded(&mut self.slider_c_value, 0.0, 30.0, id!()),
247 self.window_button(),
248 ],
249 )),
250 FlexBox::Fixed(TextInput::with_prompt(BG2, "Start typing...", input_id)),
251 // FlexBox::Fixed(Text::new("Hello world.")),
252 ],
253 ))
254 }
255
256 fn window_button(&self) -> UiRef {
257 let window_id = id!();
258 let window_button_id = id!();
259
260 let window = flat::FloatingWindow::new(
261 "Hello world",
262 window_id,
263 Col::with_gap(
264 10.0,
265 [
266 Text::body("This is a window"),
267 ImageNode::from_texture_with_scale(self.guy, 300.0),
268 Grid::with_gap(
269 4,
270 4,
271 5.0,
272 (0..16)
273 .map(|_| {
274 Border::all(
275 5.0,
276 Color::WHITE,
277 Fill::hover(Color::BLACK, Color::WHITE, EMPTY),
278 )
279 })
280 .collect::<Vec<_>>(),
281 )
282 .sized_wh(200., 200.),
283 ],
284 ),
285 );
286
287 if button_clicked_last_frame(window_button_id)
288 && let Some(state) = floating_window_state(window_id)
289 {
290 state.open = !state.open;
291 }
292
293 Stack::new([
294 flat::Button::text(flat::BG1, flat::BG2, window_button_id, "Toggle window"),
295 window,
296 ])
297 }examples/multiplayer.rs (lines 117-123)
69fn chat(state: &mut MultiplayerState<State>) {
70 if !storage_exists::<ChatState>() {
71 storage_store_state(ChatState { messages: vec![] });
72 }
73
74 let messages = &mut storage_get_state_mut::<ChatState>().messages;
75
76 let other_state = unsafe { &*{ state as *const MultiplayerState<State> } };
77 for notification in state.drain_notifications() {
78 let Some(user) = other_state.get_user(notification.user_id) else {
79 break;
80 };
81 let username = &user.username;
82 let text = String::from_utf8(notification.data).unwrap();
83
84 messages.push(rich_text(format!("<b>{username}</b>: {text}")).unwrap());
85 }
86
87 let input_id = id!();
88 let button_id = id!();
89
90 if button_clicked_last_frame(button_id) {
91 let input_state = text_input_state(input_id);
92 messages.push(rich_text(format!("<b>You</b>: {}", input_state.value)).unwrap());
93 state.send_notification(input_state.value.as_bytes().to_vec());
94 input_state.value = "".to_string();
95 }
96
97 let ui = Align::bottom_left(
98 BoxFill::new(
99 flat::BG0,
100 Padding::all(
101 20.0,
102 Col::new([
103 FlexRow::with_gap(
104 10.0,
105 [
106 FlexBox::Flex(flat::TextInput::new(flat::BG1, input_id)),
107 FlexBox::Fixed(flat::Button::text(
108 flat::BG1,
109 flat::BG2,
110 button_id,
111 "Send",
112 )),
113 ],
114 ),
115 Scroll::new(
116 id!(),
117 Col::with_gap(
118 10.0,
119 messages
120 .iter()
121 .map(|r| RichTextNode::new(r.clone()))
122 .collect::<Vec<_>>(),
123 ),
124 )
125 .padding_vertical(20.0),
126 ]),
127 ),
128 )
129 .sized_wh(600.0, 400.0),
130 )
131 .sized(window_size());
132
133 draw_ui(ui, Vec2::ZERO);
134}Trait Implementations